@@ -218,7 +218,7 @@ class OFXClient:
     appid: str = "QWIN"
     appver: str = "2700"
     language: str = "ENG"
-    useragent: str = ""
+    useragent: str = "InetClntApp/3.0"
 
     # Formatting defaults
     prettyprint: bool = False
@@ -335,14 +335,20 @@ class OFXClient:
         Package and send OFX statement requests
         (STMTRQ/CCSTMTRQ/INVSTMTRQ/STMTENDRQ/CCSTMTENDRQ).
         """
-        RqCls2url = self._get_service_urls()
+        if dryrun:
+            url = ""
+        else:
+            RqCls2url = self._get_service_urls(
+                timeout=timeout,
+                gen_newfileuid=gen_newfileuid,
+            )
 
-        # HACK FIXME
-        # As a simplification, we assume that FIs handle all classes
-        # of statement request from a single URL.
-        urls = set(RqCls2url.values())
-        assert len(urls) == 1
-        url = urls.pop()
+            # HACK FIXME
+            # As a simplification, we assume that FIs handle all classes
+            # of statement request from a single URL.
+            urls = set(RqCls2url.values())
+            assert len(urls) == 1
+            url = urls.pop()
 
         logger.info(f"Creating statement requests for {requests}")
         # Group requests by type and pass to the appropriate *TRNRQ handler
@@ -403,11 +409,18 @@ class OFXClient:
             url=url,
         )
 
-    def _get_service_urls(self) -> dict:
+    def _get_service_urls(
+        self,
+        timeout: Optional[float] = None,
+        gen_newfileuid: bool = True,
+    ) -> dict:
         """Query OFX profile endpoint to construct mapping of statement request
         data container to URL providing that service.
         """
-        profile = self.request_profile()
+        profile = self.request_profile(
+            gen_newfileuid=gen_newfileuid,
+            timeout=timeout,
+        )
         parser = OFXTree()
         parser.parse(profile)
         ofx = parser.convert()
@@ -483,9 +496,13 @@ class OFXClient:
             prettyprint=prettyprint,
             close_elements=close_elements,
             dryrun=dryrun,
-            timeout=dryrun,
+            timeout=timeout,
             url=url,
         )
+
+        if dryrun:
+            return response
+
         parser = OFXTree()
         parser.parse(response)
         ofx = parser.convert()
@@ -570,6 +587,21 @@ class OFXClient:
         """
         Package and send OFX account info requests (ACCTINFORQ)
         """
+        if dryrun:
+            url = ""
+        else:
+            RqCls2url = self._get_service_urls(
+                timeout=timeout,
+                gen_newfileuid=gen_newfileuid,
+            )
+
+            # HACK FIXME
+            # As a simplification, we assume that FIs handle all classes
+            # of statement request from a single URL.
+            urls = set(RqCls2url.values())
+            assert len(urls) == 1
+            url = urls.pop()
+
         logger.info("Creating account info request")
         signon = self.signon(password)
 
@@ -591,6 +623,7 @@ class OFXClient:
             newfileuid=newfileuid,
             dryrun=dryrun,
             timeout=timeout,
+            url=url,
         )
 
     def request_tax1099(
@@ -606,6 +639,21 @@ class OFXClient:
         """
         Request US federal income tax form 1099 (TAX1099RQ)
         """
+        if dryrun:
+            url = ""
+        else:
+            RqCls2url = self._get_service_urls(
+                timeout=timeout,
+                gen_newfileuid=gen_newfileuid,
+            )
+
+            # HACK FIXME
+            # As a simplification, we assume that FIs handle all classes
+            # of statement request from a single URL.
+            urls = set(RqCls2url.values())
+            assert len(urls) == 1
+            url = urls.pop()
+
         logger.info("Creating tax 1099 request")
         signon = self.signon(password)
 
@@ -626,6 +674,7 @@ class OFXClient:
             newfileuid=newfileuid,
             dryrun=dryrun,
             timeout=timeout,
+            url=url,
         )
 
     def signon(
@@ -765,7 +814,7 @@ class OFXClient:
         Package complete OFX tree and POST to server.
 
         N.B. ``version`` / ``prettyprint`` / ``close_elements`` kwargs are
-        basically hacks for ``scripts.ofx.scan_profile()``; ordinarily you
+        basically hacks for ``scripts.ofxget.scan_profile()``; ordinarily you
         should initialize the ``OFXClient`` with the proper version# and
         formatting parameters, rather than overriding the client config here.
 
@@ -799,7 +848,8 @@ class OFXClient:
         )
 
         if timeout in (None, False):
-            timeout = socket._GLOBAL_DEFAULT_TIMEOUT  # type: ignore
+            #  timeout = socket._GLOBAL_DEFAULT_TIMEOUT  # type: ignore
+            timeout = 10.0
 
         kwargs = dict(timeout=timeout)
 
@@ -827,7 +877,7 @@ class OFXClient:
         with OFX header prepended.
 
         N.B. ``version`` / ``prettyprint`` / ``close_elements`` kwargs are
-        basically hacks for ``scripts.ofx.scan_profile()``; ordinarily you
+        basically hacks for ``scripts.ofxget.scan_profile()``; ordinarily you
         should initialize the ``OFXClient`` with the proper version# and
         formatting parameters, rather than overriding the client config here.
 
