@@ -1,4 +1,3 @@
-# coding: utf-8
 """
 Type converters / validators for OFX data content text, used as attributes
 of OFX model classes.
@@ -33,14 +32,14 @@ __all__ = [
 
 
 # stdlib imports
-import functools
+from functools import singledispatchmethod
 import decimal
 import datetime
 import re
 import warnings
-from collections import defaultdict
 from xml.sax import saxutils
-from typing import Any, Optional
+from typing import Any, Optional, Union, Type
+import inspect
 
 
 # local imports
@@ -59,10 +58,46 @@ class OFXSpecError(OFXTypeError):
     """ Violation of the OFX specification """
 
 
-class Element:
+def call_signature(*args, **kwargs):
+    """Decorator creating ``__signature__`` class attribute (``inspect.Signature`` instance)
+    for use by ``__init__()`` method of ``Element`` subclasses.
+
+    Cf. discussion of Parameter/Signature
+    pp. 86-101 of David Beazley's "Python 3 Metaprogramming"
+    http://dabeaz.com/py3meta/Py3Meta.pdf
+
+    We don't follow Beazley's recommendation to stick the signature construction logic
+    in a custom metaclass because we're attracted to the simplicity of reusing Python's
+    *arg, **kwarg passing machinery to define the API here.
+
+    N.B. All Elements have ``required`` as keyword-only arg, with a default value of False.
     """
-    Python representation of an OFX 'element', i.e. SGML/XML leaf node that
-    contains text data.
+    #  Interpret *args as positional-only args (e.g. SubAggregate.__type__)
+    #  that are mandatory (i.e. no default)
+    params = [inspect.Parameter(arg, inspect.Parameter.POSITIONAL_ONLY) for arg in args]
+
+    # Interpret **kwargs as args that are optionally positional or keyword e.g. String.length)
+    # with the given default value
+    params += [
+        inspect.Parameter(arg, inspect.Parameter.POSITIONAL_OR_KEYWORD, default=default)
+        for arg, default in kwargs.items()
+    ]
+
+    # All Elements have ``required`` as keyword-only arg, with a default value of False
+    params.append(
+        inspect.Parameter("required", inspect.Parameter.KEYWORD_ONLY, default=False)
+    )
+
+    def decorate(cls):
+        cls.__signature__ = inspect.Signature(params)
+        return cls
+
+    return decorate
+
+
+@call_signature()
+class Element:
+    """Python representation of OFX 'element', i.e. *ML leaf node containing text data.
 
     Pass validation parameters (e.g. maximum string length, decimal scale,
     required vs. optional, etc.) as arguments to __init__() when defining
@@ -73,167 +108,147 @@ class Element:
     ``OFXHeaderV1``/``OFXHeaverV2`` classes found in the header module).
     Since these validators are class attributes, they are shared by all instances
     of a model class.  Therefore ``Elements`` are implemented as data descriptors;
-    they intercept calls to ``__get__`` and ``__set__`` redirecting them to a
-    ``defaultdict`` keyed by the calling parent, where values are the data passed
-    to that ``Element``).
+    they intercept calls to ``__get__`` and ``__set__``, which get passed as an
+    arg the ``Aggregate`` instance whose attribute you're trying to read/write.
+
+    We don't want to store the attribute value inside the ``Element`` instance, keyed by
+    the ``Aggregate`` instance, because that will cause the long-persisting ``Element``
+    to keep strong references to an ``Aggregate`` instance that may have no other
+    remaining references, thus screwing up our garbage collection & eating up memory.
+
+    Instead, we stick the attribute value where it belongs (i.e on the ``Aggregate``
+    instance), keyed by the ``Element`` instance (or even better, some proxy therefor).
+    We'll need a reference to the ``Element`` instance as long as any instance of the
+    ``Aggregate`` class remains alive, but the ``Aggregate`` instances can be garbage
+    collected when no longer needed.
+
+    A good introductory discussion to this use of descriptors is here:
+    https://realpython.com/python-descriptors/#how-to-use-python-descriptors-properly
 
     Prior to setting the data value, each ``Element`` performs validation
     (using the arguments passed to ``__init__()``) and type conversion
-    (using the logic implemented in ``_convert()``).
+    (using the logic implemented in ``convert()``).
     """
 
-    type: Any = NotImplemented  # define in subclass
+    __type__: Any = NotImplemented  # define in subclass
 
     def __init__(self, *args, **kwargs):
-        self.data = defaultdict(None)
-        self.required = kwargs.pop("required", False)
+        """ """
+        bound = self.__signature__.bind(*args, **kwargs)
+        bound.apply_defaults()
+        for name, val in bound.arguments.items():
+            setattr(self, name, val)
 
-        # N.B. ``functools.singledispatch()`` dispatches on the type of the
-        # first argument of a function, so we can't use decorator syntax on
-        # a method (unless it's a staticmethod).  Instead we use it in
-        # functional form on a bound method.
-        self.convert = functools.singledispatch(self._convert)
-        self.convert.register(type(None), self._convert_none)
-        self.convert.register(str, self._convert_str)
+    def __repr__(self) -> str:
+        repr = f"<{self.__class__.__name__}"
 
-        self.unconvert = functools.singledispatch(self._unconvert)
-        self.unconvert.register(type(None), self._unconvert_none)
+        # Mypy doesn't understand that ``__signature__`` gets set by ``__init__()``
+        for attr in self.__signature__.parameters:  # type: ignore
+            val = getattr(self, attr)
+            repr += f" {attr}={val}"
 
-        self._init(*args, **kwargs)
+        repr += ">"
+        return repr
 
-    def _init(self, *args, **kwargs):
-        """ Extend in subclass """
-        if args or kwargs:
-            cls = self.__class__.__name__
-            raise ValueError(
-                f"Unknown args for '{cls}'- args: {args}; kwargs: {kwargs}"
-            )
+    #  Descriptor protocol
+    def __set_name__(self, owner, name):
+        #  Cf. PEP 487
+        self.name = name
 
-    def __get__(self, parent, parent_type):
-        # HACK - `parent is not None` is needed for tests/test_models_base.py,
-        # else it crashes.
-        # Research!
-        if parent is not None:
-            return self.data[parent]
+    def __get__(self, obj, objtype=None):
+        """
+        ``self`` is the instance of the descriptor
+        ``obj`` is the instance of the object your descriptor is attached to.
+        ``objtype`` is the type of the object the descriptor is attached to.
+        """
+        return obj.__dict__[self.name]
 
-    def __set__(self, parent, value) -> None:
-        """ Perform validation and type conversion before setting value """
-        self.data[parent] = self.convert(value)
+    def __set__(self, obj, value) -> None:
+        """Perform validation and type conversion before setting value.
+
+        ``self`` is the instance of the descriptor
+        ``obj`` is the instance of the object your descriptor is attached to.
+        """
+        obj.__dict__[self.name] = self.convert(value)
+
+    def convert(self, value):
+        """Define in subclass"""
+        raise NotImplementedError
 
     def enforce_required(self, value):
+        """Utility used by many subclass converters"""
         if value is None and self.required:
             raise OFXSpecError(f"{self.__class__.__name__}: Value is required")
 
         return value
 
-    def _convert(self, value):
-        """ Convert OFX to Python data type """
-        return self._convert_default(value)
-
-    def _convert_default(self, value):
-        """ Default dispatch convert() for unregistered type """
-        # None should be dispatched to _convert_none()
-        assert value is not None
-        # By default, attempt a naive conversion to subclass type
-        return self.type(value)
-
-    def _convert_none(self, value):
-        """ Dispatch convert() for type(None) """
-        # Pass through None, unless value is required
-        return self.enforce_required(value)
-
-    def _convert_str(self, value):
-        """ Dispatch convert() for str """
-        # Interpret empty string as None
-        if value == "":
-            value = None
-            return self._convert_none(value)
-        return self._convert_default(value)
-
-    def _unconvert(self, value):
-        """ Convert Python data type to OFX """
-        return self._unconvert_default(value)
-
-    def _unconvert_default(self, value):
-        # By default, any type not specifically dispatched raises an error
-        cls = self.__class__.__name__
-        raise TypeError(f"{value!r} is not an instance of {cls}")
-
-    def _unconvert_none(self, value: None) -> None:
-        """ Dispatch unconvert() for type(None) """
-        # Pass through None, unless value is required
-        return self.enforce_required(value)
-
-    def __repr__(self) -> str:
-        repr = "<{} required={}>"
-        return repr.format(self.__class__.__name__, self.required)
-
 
 class Bool(Element):
-    type = bool
+    __type__ = bool
     mapping = {"Y": True, "N": False}
 
-    def _init(self, *args, **kwargs):
-        self.convert.register(bool, self._convert_bool)
-        self.unconvert.register(bool, self._unconvert_bool)
-        super()._init(*args, **kwargs)
-
-    def _convert_default(self, value):
-        # Better error message than superclass default
+    @singledispatchmethod
+    def convert(self, value):
+        # By default, any type not specifically dispatched raises an error
         msg = f"{value} is not one of the allowed values {self.mapping.keys()}"
         raise OFXSpecError(msg)
 
-    def _convert_bool(self, value):
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @convert.register
+    def _convert_bool(self, value: bool):
         return value
 
-    def _convert_str(self, value):
+    @convert.register
+    def _convert_str(self, value: str) -> bool:
         try:
-            value = self.mapping[value]
+            return self.mapping[value]
         except KeyError:
             msg = f"{value} is not one of the allowed values {self.mapping.keys()}"
             raise OFXSpecError(msg)
-        return value
 
-    def _unconvert_default(self, value):
-        # Better error message than superclass default
+    @singledispatchmethod
+    def unconvert(self, value):
         msg = f"{value} is not one of the allowed values {self.mapping.keys()}"
         raise OFXSpecError(msg)
 
-    def _unconvert_bool(self, value):
-        value = {v: k for k, v in self.mapping.items()}[value]
-        return value
+    @unconvert.register
+    def _unconvert_bool(self, value: bool) -> str:
+        return {v: k for k, v in self.mapping.items()}[value]
+
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
 
 
+@call_signature(length=None)
 class String(Element):
-    type = str
+    __type__ = str
     strict = True
 
-    def _init(self, *args, **kwargs):
-        self.unconvert.register(str, self._unconvert_str)
-
-        length = None
-        if args:
-            length = args[0]
-        self.length = length
-        super()._init(*args[1:], **kwargs)
-
-    def _convert_default(self, value):
-        # Better error message than superclass default
-        raise TypeError(f"'{value!r}' is not a str")
+    @singledispatchmethod
+    def convert(self, value):
+        # By default, any type not specifically dispatched raises an error
+        raise TypeError(f"{value!r} is not a str")
 
     def enforce_length(self, value: str) -> str:
-        if self.length is not None and len(value) > self.length:
-            msg = f"Value '{value}' exceeds max length={self.length}"
+        # Mypy doesn't understand that ``length`` gets set by ``__init__()``
+        if self.length is not None and len(value) > self.length:  # type: ignore
+            msg = f"{type(self).__name__}: {value!r} exceeds max length={self.length}"  # type: ignore
             if self.strict:
                 raise OFXSpecError(msg)
             else:
                 warnings.warn(msg, category=OFXTypeWarning)
         return value
 
-    def _convert_str(self, value):
+    @convert.register
+    def _convert_str(self, value: str) -> Optional[str]:
         if value == "":
-            value = None
-            return self.enforce_required(value)
+            return self.enforce_required(None)
 
         # Unescape '&amp;' '&lt;' '&gt;' '&nbsp;' per OFX section 2.3
         # Also go ahead and unescape other XML control characters,
@@ -241,13 +256,28 @@ class String(Element):
         value = saxutils.unescape(value, {"&nbsp;": " ", "&apos;": "'", "&quot;": '"'})
         return self.enforce_length(value)
 
-    def _unconvert_str(self, value):
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @singledispatchmethod
+    def unconvert(self, value):
+        # By default, any type not specifically dispatched raises an error
+        raise TypeError(f"{value!r} is not an instance of {self.__class__.__name__}")
+
+    @unconvert.register
+    def _unconvert_str(self, value: str):
         return self.enforce_length(value)
 
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
 
 class NagString(String):
-    """
-    String that raises a warning length is exceeded.
+    """String that raises a warning length is exceeded.
 
     Used to handle OFX data that violates the spec with respect to
     string length on non-critical fields.
@@ -257,11 +287,29 @@ class NagString(String):
 
 
 class OneOf(Element):
-    type = str
+    """Enum data type.
 
-    def _init(self, *args, **kwargs):
-        self.valid = set(args)
-        super()._init(**kwargs)
+    Usage example from ``OPTINFO``:
+    >> opttype = OneOf("CALL", "PUT", required=True)
+
+    N.B. the variable number of positional args used for instantiation violates the
+    assumptions of ``call_signature``, so we skip the ``@call_signature`` decorator
+    and directly create the ``__signature__`` attribute in the class definition.
+    """
+
+    __type__ = str
+    __signature__ = inspect.Signature(
+        (
+            inspect.Parameter("valid", kind=inspect.Parameter.VAR_POSITIONAL),
+            inspect.Parameter(
+                "required", kind=inspect.Parameter.KEYWORD_ONLY, default=False
+            ),
+        )
+    )
+
+    @singledispatchmethod
+    def convert(self, value):
+        return self._convert_default(value)
 
     def _convert_default(self, value):
         value = self.enforce_required(value)
@@ -269,89 +317,138 @@ class OneOf(Element):
             raise OFXSpecError(f"'{value}' is not OneOf {self.valid}")
         return value
 
-    def _convert_str(self, value):
-        if value == "":
-            value = None
-        return self._convert_default(value)
+    @convert.register
+    def _convert_str(self, value: str):
+        return self._convert_default(value or None)
 
-    def _unconvert_default(self, value):
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @singledispatchmethod
+    def unconvert(self, value):
         value = self.enforce_required(value)
         if value is not None and value not in self.valid:
             raise OFXSpecError(f"'{value}' is not OneOf {self.valid}")
         return value
 
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
 
+
+@call_signature(length=None)
 class Integer(Element):
-    type = int
-
-    def _init(self, *args, **kwargs):
-        self.convert.register(int, self._convert_int)
-        self.unconvert.register(int, self._unconvert_int)
-
-        length = None
-        if args:
-            length = args[0]
-        self.length = length
-
-        super()._init(*args[1:], **kwargs)
+    __type__ = int
 
     def enforce_length(self, value: int) -> int:
-        if self.length is not None and value >= 10 ** self.length:
-            msg = f"'{value}' has too many digits; max digits={self.length}"
+        # Mypy doesn't understand that ``length`` gets set by ``__init__()``
+        length = self.length  # type: ignore
+        if length is not None and value >= 10 ** length:
+            msg = f"'{value}' has too many digits; max digits={length}"
             raise OFXSpecError(msg)
         return value
 
-    def _convert_default(self, value):
-        value = int(value)
-        return self._convert_int(value)
+    @singledispatchmethod
+    def convert(self, value):
+        # By default, attempt a naive conversion to subclass type
+        return self.enforce_length(int(value))
 
-    def _convert_int(self, value):
+    @convert.register
+    def _convert_int(self, value: int):
         value = self.enforce_length(value)
         return value
 
-    def _unconvert_int(self, value):
+    @convert.register
+    def convert_str(self, value: str) -> Optional[int]:
+        if len(value) == 0:
+            return self.enforce_required(None)
+        return self.enforce_length(int(value))
+
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @singledispatchmethod
+    def unconvert(self, value):
+        # By default, any type not specifically dispatched raises an error
+        raise TypeError(f"{value!r} is not an instance of {self.__class__.__name__}")
+
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @unconvert.register
+    def _unconvert_int(self, value: int) -> str:
         value = self.enforce_length(value)
-        value = str(value)
-        return value
+        return str(value)
 
 
+#  N.B. "scale" here means "decimal places"
+#  i.e. Decimal(2).convert("12345.67890") is Decimal("12345.68")
+@call_signature(scale=None)
 class Decimal(Element):
-    type = decimal.Decimal
-    #  N.B. "scale" here means "decimal places"
-    #  i.e. Decimal(2).convert("12345.67890") is Decimal("12345.68")
-    scale = None
+    __type__ = decimal.Decimal
 
-    def _init(self, *args, **kwargs):
-        self.convert.register(decimal.Decimal, self._convert_decimal)
-        self.unconvert.register(decimal.Decimal, self._unconvert_decimal)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        #  Rewrite ``self.scale`` from # of digits to a ``decimal.Decimal`` instance
+        #  That can be directly fed into ``decimal.Decimal.quantize()``
+        if self.scale is not None:
+            self.scale = decimal.Decimal("0.{}1".format("0" * (self.scale - 1)))
 
-        if args:
-            scale = args[0]
-            self.scale = decimal.Decimal("0.{}1".format("0" * (scale - 1)))
-        super()._init(*args[1:], **kwargs)
+    @singledispatchmethod
+    def convert(self, value):
+        """ Default dispatch convert() for unregistered type """
+        # None should be dispatched to _convert_none()
+        assert value is not None
+        # By default, attempt a naive conversion to subclass type
+        return self.__type__(value)
 
-    def _convert_decimal(self, value):
+    @convert.register
+    def _convert_decimal(self, value: decimal.Decimal):
         if self.scale is not None:
             value = value.quantize(self.scale)
         return value
 
-    def _convert_str(self, value):
+    @convert.register
+    def _convert_str(self, value: str) -> decimal.Decimal:
         # Handle Euro-style decimal separators (comma)
         try:
-            value = decimal.Decimal(value)
+            dec = decimal.Decimal(value)
         except decimal.InvalidOperation:
-            value = decimal.Decimal(value.replace(",", "."))
+            dec = decimal.Decimal(value.replace(",", "."))
 
         if self.scale is not None:
-            value = value.quantize(self.scale)
+            dec = dec.quantize(self.scale)
 
-        return value
+        return dec
 
-    def _unconvert_decimal(self, value):
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @singledispatchmethod
+    def unconvert(self, value):
+        # By default, any type not specifically dispatched raises an error
+        raise TypeError(f"{value!r} is not an instance of {self.__class__.__name__}")
+
+    @unconvert.register
+    def _unconvert_decimal(self, value: decimal.Decimal):
         if self.scale is not None and not value.same_quantum(self.scale):
             raise ValueError(f"'{value}' doesn't match scale={self.scale}")
         return str(value)
 
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
 
 # Valid datetime formats per the OFX spec (in OFX_Common.xsd):
 #  [0-9]{4}
@@ -418,27 +515,26 @@ DT_REGEX = re.compile(
 class DateTime(Element):
     """ OFX Section 3.2.8.2 """
 
-    type: Any = datetime.datetime
+    # __type__ must be compatible with Time subclass override
+    __type__: Union[Type[datetime.datetime], Type[datetime.time]] = datetime.datetime
     regex = DT_REGEX
 
-    def _init(self, *args, **kwargs):
-        self.convert.register(datetime.datetime, self._convert_datetime)
-        self.unconvert.register(datetime.datetime, self._unconvert_datetime)
-        super()._init(*args, **kwargs)
-
-    def _convert_default(self, value):
+    @singledispatchmethod
+    def convert(self, value):
         cls = value.__class__.__name__
-        raise TypeError(f"{value!r} is type '{cls}'; can't convert to {self.type}")
+        raise TypeError(f"{value!r} is type '{cls}'; can't convert to {self.__type__}")
 
-    def _convert_datetime(self, value):
+    @convert.register
+    def _convert_datetime(self, value: datetime.datetime):
         if value.utcoffset() is None:
             raise ValueError(f"{value} is not timezone-aware")
         return value
 
-    def _convert_str(self, value):
+    @convert.register
+    def _convert_str(self, value: str):
         match = self.regex.match(value)
         if match is None:
-            msg = f"'{value}' does not conform to OFX formats for {self.type}"
+            msg = f"'{value}' does not conform to OFX formats for {self.__type__}"
             raise OFXSpecError(msg)
 
         matchdict = match.groupdict()
@@ -454,7 +550,8 @@ class DateTime(Element):
         # OFX time formats give milliseconds,
         # but datetime.datetime wants microseconds
         intmatches["microsecond"] = 1000 * intmatches.pop("millisecond")
-        return self.normalize_to_gmt(self.type(**intmatches), gmt_offset)
+        # Mypy doesn't understand passing ``**intmatches`` to instantiate
+        return self.normalize_to_gmt(self.__type__(**intmatches), gmt_offset)  # type: ignore
 
     def parse_gmt_offset(
         self, hours: Optional[str], minutes: Optional[str], tz_name: Optional[str]
@@ -478,9 +575,20 @@ class DateTime(Element):
         self.unconvert.register(datetime.datetime, self._unconvert_datetime)
         return (value - gmt_offset).replace(tzinfo=utils.UTC)
 
-    def _unconvert_datetime(self, value):
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
+    @singledispatchmethod
+    def unconvert(self, value):
+        # By default, any type not specifically dispatched raises an error
+        raise TypeError(f"{value!r} is not an instance of {self.__class__.__name__}")
+
+    @unconvert.register
+    def _unconvert_datetime(self, value: datetime.datetime):
         if not hasattr(value, "utcoffset") or value.utcoffset() is None:
-            msg = f"'{value}' isn't a timezone-aware {self.type} instance; can't convert to GMT"
+            msg = f"'{value}' isn't a timezone-aware {self.__type__} instance; can't convert to GMT"
             raise ValueError(msg)
 
         # Transform to GMT
@@ -500,6 +608,11 @@ class DateTime(Element):
         fmt = "%Y%m%d%H%M%S.{}[0:GMT]".format(millisec_str)
         return value.strftime(fmt)
 
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
 
 # Valid time formats given by OFX spec (in OFX_Common.xsd):
 #  (([0-1][0-9])|(2[0-3]))
@@ -538,14 +651,19 @@ TIME_REGEX = re.compile(
 class Time(DateTime):
     """ OFX Section 3.2.8.3 """
 
-    type = datetime.time
+    __type__ = datetime.time
     regex = TIME_REGEX
 
-    def _init(self, *args, **kwargs):
-        self.convert.register(datetime.time, self._convert_time)
-        self.unconvert.register(datetime.time, self._unconvert_time)
+    @singledispatchmethod
+    def convert(self, value):
+        cls = value.__class__.__name__
+        raise TypeError(f"{value!r} is type '{cls}'; can't convert to {self.__type__}")
 
-        super()._init(*args, **kwargs)
+    @convert.register
+    def _convert_time(self, value: datetime.time):
+        if value.utcoffset() is None:
+            raise ValueError(f"{value} is not timezone-aware")
+        return value
 
     def normalize_to_gmt(
         self, value: datetime.time, gmt_offset: datetime.timedelta
@@ -563,17 +681,24 @@ class Time(DateTime):
         )
         return (dt - gmt_offset).time().replace(tzinfo=utils.UTC)
 
-    def _convert_time(self, value):
-        if value.utcoffset() is None:
-            raise ValueError(f"{value} is not timezone-aware")
-        return value
+    @convert.register
+    def _convert_str(self, value: str):
+        return super()._convert_str(value)  # type: ignore
 
-    def _convert_datetime(self, value):
-        return self._convert_default(value)
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
 
-    def _unconvert_time(self, value):
+    @singledispatchmethod
+    def unconvert(self, value):
+        # By default, any type not specifically dispatched raises an error
+        raise TypeError(f"{value!r} is not an instance of {self.__class__.__name__}")
+
+    @unconvert.register
+    def _unconvert_time(self, value: datetime.time):
         if not hasattr(value, "utcoffset") or value.utcoffset() is None:
-            msg = f"'{value}' isn't a timezone-aware {self.type} instance; can't convert to GMT"
+            msg = f"'{value}' isn't a timezone-aware {self.__type__} instance; can't convert to GMT"
             raise ValueError(msg)
 
         # Transform to GMT
@@ -586,15 +711,18 @@ class Time(DateTime):
             value.second,
             microsecond=value.microsecond,
         )
-        dt -= value.utcoffset()
+        dt -= value.utcoffset()  # type: ignore
         milliseconds = "{0:03d}".format((dt.microsecond + 500) // 1000)
         fmt = "%H%M%S.{}[0:GMT]".format(milliseconds)
         return dt.strftime(fmt)
 
-    def _unconvert_datetime(self, value):
-        return self._unconvert_default(value)
+    @unconvert.register
+    def _unconvert_none(self, value: None) -> None:
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
 
 
+@call_signature("converter")
 class ListElement(Element):
     """
     ``Element`` that can be repeated on the parent ``Aggregate``.
@@ -605,17 +733,14 @@ class ListElement(Element):
         ``ListElement(String(32))``
     """
 
-    def _init(self, *args, **kwargs):
-        self.converter = args[0]
-        super()._init(*args[1:], **kwargs)
-
-    def _convert_default(self, value):
+    def convert(self, value):
         return self.converter.convert(value)
 
-    def _unconvert_default(self, value):
+    def unconvert(self, value):
         return self.converter.unconvert(value)
 
 
+@call_signature("__type__")
 class SubAggregate(Element):
     """
     Parent/child relationship between ``Aggregates`` - used for child ``Aggregates``
@@ -631,18 +756,20 @@ class SubAggregate(Element):
         ``SubAggregate(BANKACCTFROM, required=True)``
     """
 
-    def _init(self, *args, **kwargs):
-        self.type = args[0]
-        super()._init(*args[1:], **kwargs)
-
-    def _convert_default(self, value):
-        if not isinstance(value, self.type):
-            raise TypeError(f"'{value}' is not an instance of {self.type}")
+    @singledispatchmethod
+    def convert(self, value):
+        if not isinstance(value, self.__type__):
+            raise TypeError(f"'{value}' is not an instance of {self.__type__}")
         return value
 
+    @convert.register
+    def _convert_none(self, value: None):
+        # Pass through None, unless value is required
+        return self.enforce_required(value)
+
     #  This doesn't get used
     #  def __repr__(self):
-    #  return "<{}>".format(self.type.__name__)
+    #  return "<{}>".format(self.__type__.__name__)
 
 
 class ListAggregate(SubAggregate):
@@ -650,9 +777,9 @@ class ListAggregate(SubAggregate):
     ``SubAggregate`` that can be repeated on the parent ``Aggregate``.
     """
 
-    def _unconvert_default(self, value):
-        if not isinstance(value, self.type):
-            raise TypeError(f"'{value!r}' is not an instance of {self.type}")
+    def unconvert(self, value):
+        if not isinstance(value, self.__type__):
+            raise TypeError(f"'{value!r}' is not an instance of {self.__type__}")
         return value
 
 
@@ -661,10 +788,10 @@ class Unsupported:
     Null Aggregate/Element - not implemented (yet)
     """
 
-    def __get__(self, instance, type_) -> None:
+    def __get__(self, obj, objtype) -> None:
         pass
 
-    def __set__(self, instance, value) -> None:
+    def __set__(self, obj, value) -> None:
         pass
 
     def __repr__(self) -> str:
